@@ -1,5 +1,4 @@
 import sys
-from collections import defaultdict
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -11,15 +10,19 @@ from typing import (
     Union,
 )
 
-from dvc.progress import Tqdm
-from dvc.utils import colorize
+from funcy import cached_property
 
 if TYPE_CHECKING:
+    from rich.status import Status
+
+    from dvc.progress import Tqdm
     from dvc.ui.table import Headers, Styles, TableData
 
 
 class Formatter:
     def __init__(self, theme: Dict = None, defaults: Dict = None) -> None:
+        from collections import defaultdict
+
         theme = theme or {
             "success": {"color": "green", "style": "bold"},
             "warn": {"color": "yellow"},
@@ -28,30 +31,18 @@ class Formatter:
         self.theme = defaultdict(lambda: defaults or {}, theme)
 
     def format(self, message: str, style: str = None, **kwargs) -> str:
+        from dvc.utils import colorize
+
         return colorize(message, **self.theme[style])
 
 
 class Console:
     def __init__(
-        self,
-        formatter: Formatter = None,
-        output: TextIO = None,
-        error: TextIO = None,
-        enable: bool = False,
+        self, formatter: Formatter = None, enable: bool = False
     ) -> None:
-        self._output: Optional[TextIO] = output
-        self._error: Optional[TextIO] = error
         self.formatter: Formatter = formatter or Formatter()
         self._enabled: bool = enable
 
-    @property
-    def output(self) -> TextIO:
-        return self._output or sys.stdout
-
-    @property
-    def error_output(self) -> TextIO:
-        return self._error or sys.stderr
-
     def enable(self) -> None:
         self._enabled = True
 
@@ -70,15 +61,13 @@ class Console:
         style: str = None,
         sep: str = None,
         end: str = None,
-        flush: bool = False,
     ) -> None:
         return self.write(
             *objects,
             style=style,
             sep=sep,
             end=end,
-            file=self.error_output,
-            flush=flush,
+            stderr=True,
         )
 
     def write(
@@ -87,19 +76,33 @@ class Console:
         style: str = None,
         sep: str = None,
         end: str = None,
-        file: TextIO = None,
-        flush: bool = False,
+        stderr: bool = False,
         force: bool = False,
+        styled: bool = False,
+        file: TextIO = None,
     ) -> None:
+        from dvc.progress import Tqdm
+
+        sep = " " if sep is None else sep
+        end = "\n" if end is None else end
         if not self._enabled and not force:
             return
 
-        file = file or self.output
-        values = (self.formatter.format(obj, style=style) for obj in objects)
-        return print(*values, sep=sep, end=end, file=file, flush=flush)
+        file = file or (sys.stderr if stderr else sys.stdout)
+        with Tqdm.external_write_mode(file=file):
+            if styled:
+                console = self.error_console if stderr else self.rich_console
+                return console.print(*objects, sep=sep, end=end)
+
+            values = (
+                self.formatter.format(obj, style=style) for obj in objects
+            )
+            return print(*values, sep=sep, end=end, file=file)
+
+    @staticmethod
+    def progress(*args, **kwargs) -> "Tqdm":
+        from dvc.progress import Tqdm
 
-    def progress(self, *args, **kwargs) -> Tqdm:
-        kwargs.setdefault("file", self.error_output)
         return Tqdm(*args, **kwargs)
 
     def prompt(
@@ -137,15 +140,18 @@ class Console:
             return False
         return answer.startswith("y")
 
-    @property
+    @cached_property
     def rich_console(self):
         """rich_console is only set to stdout for now."""
         from rich import console
 
-        # FIXME: Getting IO Operation on closed file error
-        #  when testing with capsys, therefore we are creating
-        #  one instance each time as a temporary workaround.
-        return console.Console(file=self.output)
+        return console.Console()
+
+    @cached_property
+    def error_console(self):
+        from rich import console
+
+        return console.Console(stderr=True)
 
     def table(
         self,
@@ -155,7 +161,7 @@ class Console:
         rich_table: bool = False,
         force: bool = True,
         pager: bool = False,
-        header_styles: Sequence["Styles"] = None,
+        header_styles: Union[Dict[str, "Styles"], Sequence["Styles"]] = None,
         row_styles: Sequence["Styles"] = None,
         borders: Union[bool, str] = False,
     ) -> None:
@@ -179,9 +185,12 @@ class Console:
             return
 
         return t.plain_table(
-            self, data, headers, markdown=markdown, pager=pager, force=force,
+            self, data, headers, markdown=markdown, pager=pager, force=force
         )
 
+    def status(self, status: str, **kwargs: Any) -> "Status":
+        return self.error_console.status(status, **kwargs)
+
 
 ui = Console()
 
